@@ -1,10 +1,10 @@
 import json
-from contextlib import nullcontext
 from unittest.mock import patch
 
 import pytest
 from tornado.httpclient import HTTPClientError
 
+from jupyter_scheduler.exceptions import IdempotencyTokenError, InputUriError
 from jupyter_scheduler.handlers import compute_sort_model
 from jupyter_scheduler.models import (
     CountJobsQuery,
@@ -45,9 +45,9 @@ from jupyter_scheduler.tests.utils import expected_http_error
         ),
     ],
 )
-async def test_post_scheduled_jobs(jp_fetch, job_id, payload):
-    with patch("jupyter_scheduler.orm.uuid4") as mock_uuid:
-        mock_uuid.return_value = job_id
+async def test_post_jobs(jp_fetch, job_id, payload):
+    with patch("jupyter_scheduler.scheduler.Scheduler.create_job") as mock_create_job:
+        mock_create_job.return_value = job_id
         response = await jp_fetch("scheduler", "jobs", method="POST", body=json.dumps(payload))
 
         assert response.code == 200
@@ -55,6 +55,56 @@ async def test_post_scheduled_jobs(jp_fetch, job_id, payload):
         assert body["job_id"] == job_id
 
 
+async def test_post_jobs_for_invalid_input_uri(jp_fetch):
+    payload = {
+        "name": "job_a",
+        "input_uri": "notebook_a.ipynb",
+        "output_prefix": "outputs",
+        "runtime_environment_name": "env_a",
+        "idempotency_token": "a",
+    }
+    input_path = payload["input_uri"]
+    with patch("jupyter_scheduler.scheduler.Scheduler.create_job") as mock_create_job:
+        mock_create_job.side_effect = InputUriError(input_path)
+        with pytest.raises(HTTPClientError) as e:
+            await jp_fetch("scheduler", "jobs", method="POST", body=json.dumps(payload))
+
+        assert expected_http_error(e, 500, f"Input path '{input_path}' does not exist.")
+
+
+async def test_post_jobs_for_idempotency_token_error(jp_fetch):
+    payload = {
+        "name": "job_a",
+        "input_uri": "notebook_a.ipynb",
+        "output_prefix": "outputs",
+        "runtime_environment_name": "env_a",
+        "idempotency_token": "a",
+    }
+    idempotency_token = payload["idempotency_token"]
+    with patch("jupyter_scheduler.scheduler.Scheduler.create_job") as mock_create_job:
+        mock_create_job.side_effect = IdempotencyTokenError(idempotency_token)
+        with pytest.raises(HTTPClientError) as e:
+            await jp_fetch("scheduler", "jobs", method="POST", body=json.dumps(payload))
+        assert expected_http_error(
+            e, 409, f"Job with Idempotency Token '{idempotency_token}' already exists."
+        )
+
+
+async def test_post_jobs_for_unexpected_error(jp_fetch):
+    payload = {
+        "name": "job_a",
+        "input_uri": "notebook_a.ipynb",
+        "output_prefix": "outputs",
+        "runtime_environment_name": "env_a",
+        "idempotency_token": "a",
+    }
+    with patch("jupyter_scheduler.scheduler.Scheduler.create_job") as mock_create_job:
+        mock_create_job.side_effect = Exception("Unexpected error")
+        with pytest.raises(HTTPClientError) as e:
+            await jp_fetch("scheduler", "jobs", method="POST", body=json.dumps(payload))
+        assert expected_http_error(e, 500, "Unexpected error occurred during creation of Job.")
+
+
 async def test_get_jobs_for_single_job(jp_fetch):
     with patch("jupyter_scheduler.scheduler.Scheduler.get_job") as mock_get_job:
         job_id = "542e0fac-1274-4a78-8340-a850bdb559c8"
@@ -63,7 +113,13 @@ async def test_get_jobs_for_single_job(jp_fetch):
             output_prefix="output_a",
             runtime_environment_name="environment_a",
             job_id=job_id,
-            output_uri="output_a/input_a_12345",
+            outputs=[
+                {
+                    "display_name": "Notebook",
+                    "output_format": "ipynb",
+                    "output_path": "output_a/input_a_12345",
+                }
+            ],
             url="url_a",
             create_time=1664305872620,
             update_time=1664305872620,
@@ -75,7 +131,7 @@ async def test_get_jobs_for_single_job(jp_fetch):
         body = json.loads(response.body)
         assert body["job_id"] == job_id
         assert body["input_uri"]
-        assert body["output_uri"]
+        assert body["outputs"]
 
 
 @pytest.mark.parametrize(
@@ -91,7 +147,13 @@ async def test_get_jobs_for_single_job(jp_fetch):
                         "output_prefix": "output_a",
                         "runtime_environment_name": "environment_a",
                         "job_id": "542e0fac-1274-4a78-8340-a850bdb559c8",
-                        "output_uri": "output_a/input_a_12345",
+                        "outputs": [
+                            {
+                                "display_name": "Notebook",
+                                "output_format": "ipynb",
+                                "output_path": "output_a/input_a_12345",
+                            }
+                        ],
                         "url": "url_a",
                         "create_time": 1664305872620,
                         "update_time": 1664305872620,
@@ -132,7 +194,13 @@ async def test_get_jobs_for_single_job(jp_fetch):
                         "output_prefix": "output_a",
                         "runtime_environment_name": "environment_a",
                         "job_id": "542e0fac-1274-4a78-8340-a850bdb559c8",
-                        "output_uri": "output_a/input_a_12345",
+                        "outputs": [
+                            {
+                                "display_name": "Notebook",
+                                "output_format": "ipynb",
+                                "output_path": "output_a/input_a_12345",
+                            }
+                        ],
                         "url": "url_a",
                         "create_time": 1664305872620,
                         "update_time": 1664305872620,
@@ -159,7 +227,7 @@ async def test_get_jobs(jp_fetch, params, list_query, jobs_list):
         assert actual_job["output_prefix"] == expected_job["output_prefix"]
         assert actual_job["runtime_environment_name"] == expected_job["runtime_environment_name"]
         assert actual_job["job_id"] == expected_job["job_id"]
-        assert actual_job["output_uri"] == expected_job["output_uri"]
+        assert actual_job["outputs"] == expected_job["outputs"]
         assert actual_job["url"] == expected_job["url"]
 
 
@@ -179,11 +247,12 @@ async def test_patch_jobs_for_invalid_status(jp_fetch):
         job_id = "542e0fac-1274-4a78-8340-a850bdb559c8"
         body = {"status": "IN_PROGRESS"}
         await jp_fetch("scheduler", "jobs", job_id, method="PATCH", body=json.dumps(body))
-        assert expected_http_error(
-            e,
-            500,
-            "Invalid value for field 'status'. Jobs can only be updated to status 'STOPPED' after creation.",
-        )
+
+    assert expected_http_error(
+        e,
+        500,
+        "Invalid value for field 'status'. Jobs can only be updated to status 'STOPPED' after creation.",
+    )
 
 
 async def test_patch_jobs(jp_fetch):
@@ -208,13 +277,11 @@ async def test_patch_jobs_for_stop_job(jp_fetch):
         assert response.code == 204
 
 
-# TODO: Enable after fixing patch api
-@pytest.mark.skip
 async def test_patch_jobs_for_name_update(jp_fetch):
     with patch("jupyter_scheduler.scheduler.Scheduler.stop_job") as mock_stop_job:
         job_id = "542e0fac-1274-4a78-8340-a850bdb559c8"
         response = await jp_fetch(
-            "scheduler", "jobs", job_id, method="PATCH", body=json.dumps({"name": "New job name"})
+            "scheduler", "jobs", job_id, method="PATCH", body=json.dumps({"status": "STOPPED"})
         )
 
         mock_stop_job.assert_called_once_with(job_id)
