@@ -48,6 +48,14 @@ type AdvancedTableProps<
   columns: AdvancedTableColumn[];
   emptyRowMessage: string;
   rowFilter?: (row: R) => boolean;
+  pageSize?: number;
+  /**
+   * Height of the table. If set to 'auto', this table automatically expands to
+   * fit the remaining height of the page when wrapped in a flex container with
+   * its height equal to the height of the page. Otherwise the max-height is
+   * manually set to whatever value is provided. Defaults to 'auto'.
+   */
+  height?: 'auto' | string | number;
 };
 
 /**
@@ -67,6 +75,8 @@ export function AdvancedTable<
   const [loading, setLoading] = useState<boolean>(true);
   const theme = useTheme();
 
+  const pageSize = props.pageSize ?? PAGE_SIZE;
+
   const fetchInitialRows = async () => {
     // reset pagination state
     setPage(0);
@@ -75,7 +85,7 @@ export function AdvancedTable<
     setLoading(true);
     const payload = await props.request({
       ...props.query,
-      max_items: PAGE_SIZE
+      max_items: pageSize
     });
     setLoading(false);
 
@@ -105,7 +115,7 @@ export function AdvancedTable<
     setLoading(true);
     const payload = await props.request({
       ...props.query,
-      max_items: PAGE_SIZE,
+      max_items: pageSize,
       next_token: nextToken
     });
     setLoading(false);
@@ -127,7 +137,7 @@ export function AdvancedTable<
   }
 
   const renderedRows: JSX.Element[] = (rows || [])
-    .slice(page * PAGE_SIZE, (page + 1) * PAGE_SIZE)
+    .slice(page * pageSize, (page + 1) * pageSize)
     .filter(row => (props.rowFilter ? props.rowFilter(row) : true))
     .map(row => props.renderRow(row));
 
@@ -143,9 +153,12 @@ export function AdvancedTable<
     setMaxPage(newPage);
   };
 
-  // outer div expands to fill rest of screen
+  const height = props.height ?? 'auto';
+
   return (
-    <div style={{ flex: 1, height: 0 }}>
+    <div
+      style={height === 'auto' ? { flex: 1, height: 0 } : { maxHeight: height }}
+    >
       <TableContainer
         component={Paper}
         sx={{
@@ -175,8 +188,8 @@ export function AdvancedTable<
           nextIconButtonProps={{
             disabled: page === maxPage && !nextToken
           }}
-          rowsPerPage={PAGE_SIZE}
-          rowsPerPageOptions={[PAGE_SIZE]}
+          rowsPerPage={pageSize}
+          rowsPerPageOptions={[pageSize]}
         />
       </TableContainer>
     </div>
