@@ -1,23 +1,21 @@
-import React from 'react';
+import React, { useState } from 'react';
 
 import { JupyterFrontEnd } from '@jupyterlab/application';
 import { PathExt } from '@jupyterlab/coreutils';
 
-import CloseIcon from '@mui/icons-material/Close';
+import DownloadIcon from '@mui/icons-material/Download';
 import StopIcon from '@mui/icons-material/Stop';
 import ReplayIcon from '@mui/icons-material/Replay';
-
 import IconButton from '@mui/material/IconButton';
 import Stack from '@mui/material/Stack';
-import TableRow from '@mui/material/TableRow';
-import TableCell from '@mui/material/TableCell';
-
+import { outputFormatsForEnvironment } from './output-format-picker';
 import { Scheduler } from '../handler';
 import { useTranslator } from '../hooks';
 import { IJobParameter, ICreateJobModel } from '../model';
 import { CommandIDs } from '..';
-
-import { outputFormatsForEnvironment } from './output-format-picker';
+import { ConfirmDeleteIcon } from './confirm-delete-icon';
+import TableRow from '@mui/material/TableRow';
+import { TableCell } from '@mui/material';
 
 function get_file_from_path(path: string): string {
   return PathExt.basename(path);
@@ -43,22 +41,6 @@ function StopButton(props: {
   );
 }
 
-function DeleteButton(props: {
-  job: Scheduler.IDescribeJob;
-  clickHandler: () => void;
-}): JSX.Element | null {
-  const trans = useTranslator('jupyterlab');
-  const buttonTitle = props.job.name
-    ? trans.__('Delete "%1"', props.job.name)
-    : trans.__('Delete job');
-
-  return (
-    <IconButton onClick={props.clickHandler} title={buttonTitle}>
-      <CloseIcon fontSize="small" />
-    </IconButton>
-  );
-}
-
 function RefillButton(props: {
   job: Scheduler.IDescribeJob;
   environmentList: Scheduler.IRuntimeEnvironment[];
@@ -140,29 +122,59 @@ function OutputFiles(props: {
 
   const trans = useTranslator('jupyterlab');
 
-  // Get all output files.
-  const outputTypes = props.job.output_formats || ['ipynb'];
   return (
     <>
-      {outputTypes.map(outputType => {
-        // Compose a specific link.
-        const outputName = props.job.output_uri.replace(/ipynb$/, outputType);
-        return (
-          <a
-            key={outputType}
-            href={`/lab/tree/${outputName}`}
-            title={trans.__('Open "%1"', outputName)}
-            onClick={e => props.openOnClick(e, outputName)}
-            style={{ paddingRight: '1em' }}
-          >
-            {outputType}
-          </a>
-        );
-      })}
+      {props.job.outputs.map(
+        ({ output_format, display_name, output_path = null }) => {
+          return (
+            output_path && (
+              <a
+                key={output_format}
+                href={`/lab/tree/${output_path}`}
+                title={trans.__('Open "%1"', output_path)}
+                onClick={e => props.openOnClick(e, output_path)}
+                style={{ paddingRight: '1em' }}
+              >
+                {display_name}
+              </a>
+            )
+          );
+        }
+      )}
     </>
   );
 }
 
+type DownloadOutputsButtonProps = {
+  app: JupyterFrontEnd;
+  job: Scheduler.IDescribeJob;
+  reload: () => void;
+};
+
+function DownloadOutputsButton(props: DownloadOutputsButtonProps) {
+  const [downloading, setDownloading] = useState(false);
+
+  return (
+    <IconButton
+      aria-label="download"
+      title="Download Output Files"
+      disabled={downloading}
+      onClick={async () => {
+        setDownloading(true);
+        await props.app.commands.execute(CommandIDs.downloadOutputs, {
+          id: props.job.job_id,
+          redownload: false
+        });
+        await new Promise(res => setTimeout(res, 500));
+        setDownloading(false);
+        props.reload();
+      }}
+    >
+      <DownloadIcon />
+    </IconButton>
+  );
+}
+
 export function buildJobRow(
   job: Scheduler.IDescribeJob,
   environmentList: Scheduler.IRuntimeEnvironment[],
@@ -170,18 +182,24 @@ export function buildJobRow(
   showCreateJob: (newModel: ICreateJobModel) => void,
   deleteRow: (id: Scheduler.IDescribeJob['job_id']) => void,
   translateStatus: (status: Scheduler.Status) => string,
-  showDetailView: (jobId: string) => void
+  showDetailView: (jobId: string) => void,
+  reload: () => void
 ): JSX.Element {
   const cellContents: React.ReactNode[] = [
     <a onClick={() => showDetailView(job.job_id)}>{job.name}</a>,
     get_file_from_path(job.input_uri),
-    <OutputFiles
-      job={job}
-      openOnClick={(e: Event, output_uri: string) => {
-        e.preventDefault();
-        app.commands.execute('docmanager:open', { path: output_uri });
-      }}
-    />,
+    <>
+      {!job.downloaded && job.status === 'COMPLETED' && (
+        <DownloadOutputsButton app={app} job={job} reload={reload} />
+      )}
+      <OutputFiles
+        job={job}
+        openOnClick={(e: Event, output_uri: string) => {
+          e.preventDefault();
+          app.commands.execute('docmanager:open', { path: output_uri });
+        }}
+      />
+    </>,
     <Timestamp job={job} />,
     translateStatus(job.status),
     <Stack spacing={1} direction="row">
@@ -193,8 +211,13 @@ export function buildJobRow(
           })
         }
       />
-      <DeleteButton
+      <RefillButton
         job={job}
+        environmentList={environmentList}
+        showCreateJob={showCreateJob}
+      />
+      <ConfirmDeleteIcon
+        name={job.name}
         clickHandler={() => {
           // optimistic delete for now, no verification on whether the delete
           // succeeded
@@ -204,18 +227,13 @@ export function buildJobRow(
           deleteRow(job.job_id);
         }}
       />
-      <RefillButton
-        job={job}
-        environmentList={environmentList}
-        showCreateJob={showCreateJob}
-      />
     </Stack>
   ];
 
   return (
-    <TableRow>
+    <TableRow key={job.job_id}>
       {cellContents.map((cellContent, idx) => (
-        <TableCell key={idx}>{cellContent}</TableCell>
+        <TableCell key={`${job.job_id}-${idx}`}>{cellContent}</TableCell>
       ))}
     </TableRow>
   );
