@@ -11,6 +11,7 @@ import {
 } from '@jupyterlab/apputils';
 import { FileBrowser, IFileBrowserFactory } from '@jupyterlab/filebrowser';
 import { ILauncher } from '@jupyterlab/launcher';
+import { INotebookTracker } from '@jupyterlab/notebook';
 import { Contents } from '@jupyterlab/services';
 import { IStatusBar } from '@jupyterlab/statusbar';
 import { ITranslator } from '@jupyterlab/translation';
@@ -19,7 +20,11 @@ import { Poll } from '@lumino/polling';
 import { RunningJobsIndicator } from './components/running-jobs-indicator';
 
 import { SchedulerService } from './handler';
-import { JobsView, ICreateJobModel, NotebookJobsListingModel } from './model';
+import {
+  NotebookJobsListingModel,
+  IJobsModel,
+  emptyCreateJobModel
+} from './model';
 import { NotebookJobsPanel } from './notebook-jobs-panel';
 import {
   calendarAddOnIcon,
@@ -31,9 +36,11 @@ import AdvancedOptions from './advanced-options';
 
 export namespace CommandIDs {
   export const deleteJob = 'scheduling:delete-job';
-  export const runNotebook = 'scheduling:run-notebook';
+  export const createJobFileBrowser = 'scheduling:create-from-filebrowser';
+  export const createJobCurrentNotebook = 'scheduling:create-from-notebook';
   export const showNotebookJobs = 'scheduling:show-notebook-jobs';
   export const stopJob = 'scheduling:stop-job';
+  export const downloadOutputs = 'scheduling:download-outputs';
 }
 
 export const NotebookJobsPanelId = 'notebook-jobs-panel';
@@ -46,6 +53,7 @@ const schedulerPlugin: JupyterFrontEndPlugin<void> = {
   id: '@jupyterlab/scheduler:plugin',
   requires: [
     IFileBrowserFactory,
+    INotebookTracker,
     ITranslator,
     ILayoutRestorer,
     Scheduler.IAdvancedOptions
@@ -112,6 +120,7 @@ async function getNotebookJobsListingModel(): Promise<NotebookJobsListingModel>
 async function activatePlugin(
   app: JupyterFrontEnd,
   browserFactory: IFileBrowserFactory,
+  notebookTracker: INotebookTracker,
   translator: ITranslator,
   restorer: ILayoutRestorer,
   advancedOptions: Scheduler.IAdvancedOptions,
@@ -125,29 +134,21 @@ async function activatePlugin(
 
   const { commands } = app;
   const trans = translator.load('jupyterlab');
-  const { tracker } = browserFactory;
+  const fileBrowserTracker = browserFactory.tracker;
   const api = new SchedulerService({});
   const widgetTracker = new WidgetTracker<MainAreaWidget<NotebookJobsPanel>>({
     namespace: 'jupyterlab-scheduler'
   });
   restorer.restore(widgetTracker, {
     command: CommandIDs.showNotebookJobs,
+    args: widget => widget.content.model.toJson(),
     name: () => 'jupyterlab-scheduler'
   });
 
-  commands.addCommand(CommandIDs.deleteJob, {
-    execute: async args => {
-      const id = args['id'] as string;
-      await api.deleteJob(id);
-    },
-    // TODO: Use args to name command dynamically
-    label: trans.__('Delete Job')
-  });
-
   let mainAreaWidget: MainAreaWidget<NotebookJobsPanel> | undefined;
   let jobsPanel: NotebookJobsPanel | undefined;
 
-  const showJobsPane = async (view: JobsView) => {
+  const showJobsPanel = async (data: IJobsModel) => {
     if (!mainAreaWidget || mainAreaWidget.isDisposed) {
       // Create new jobs panel widget
       jobsPanel = new NotebookJobsPanel({
@@ -159,7 +160,6 @@ async function activatePlugin(
       mainAreaWidget = new MainAreaWidget<NotebookJobsPanel>({
         content: jobsPanel
       });
-      mainAreaWidget.content.model.jobsView = view;
       mainAreaWidget.id = NotebookJobsPanelId;
       mainAreaWidget.title.icon = calendarMonthIcon;
       mainAreaWidget.title.label = trans.__('Notebook Jobs');
@@ -169,53 +169,81 @@ async function activatePlugin(
     if (!widgetTracker.has(mainAreaWidget)) {
       // Track the state of the widget for later restoration
       widgetTracker.add(mainAreaWidget);
+      mainAreaWidget.content.model.stateChanged.connect(() => {
+        void widgetTracker.save(
+          mainAreaWidget as MainAreaWidget<NotebookJobsPanel>
+        );
+      });
     }
 
     if (!mainAreaWidget.isAttached) {
       app.shell.add(mainAreaWidget, 'main');
     }
 
-    mainAreaWidget.content.model.jobsView = view;
+    mainAreaWidget.content.model.fromJson(data);
     mainAreaWidget.content.update();
     app.shell.activateById(mainAreaWidget.id);
   };
 
+  // Commands
+
   commands.addCommand(CommandIDs.showNotebookJobs, {
-    execute: async () => showJobsPane('ListJobs'),
+    execute: async args => showJobsPanel(args as IJobsModel),
     label: trans.__('Notebook Jobs'),
     icon: eventNoteIcon
   });
 
-  commands.addCommand(CommandIDs.runNotebook, {
+  commands.addCommand(CommandIDs.createJobFileBrowser, {
     execute: async () => {
-      await showJobsPane('CreateJob');
-
-      const model = jobsPanel?.model;
-      if (!model) {
-        return;
-      }
-
-      const widget = tracker.currentWidget;
+      const widget = fileBrowserTracker.currentWidget;
       const filePath = getSelectedFilePath(widget) ?? '';
       const fileName = getSelectedFileName(widget) ?? '';
 
       // Update the job form inside the notebook jobs widget
-      const newModel: ICreateJobModel = {
-        inputFile: filePath,
-        jobName: fileName,
-        outputPath: '',
-        environment: '',
-        createType: 'Job',
-        scheduleInterval: 'weekday',
-        timezone: Intl.DateTimeFormat().resolvedOptions().timeZone
-      };
+      const newCreateModel = emptyCreateJobModel();
+      newCreateModel.inputFile = filePath;
+      newCreateModel.jobName = fileName;
 
-      model.createJobModel = newModel;
+      await showJobsPanel({
+        jobsView: 'CreateJob',
+        createJobModel: newCreateModel
+      });
     },
     label: trans.__('Create Notebook Job'),
     icon: calendarAddOnIcon
   });
 
+  commands.addCommand(CommandIDs.createJobCurrentNotebook, {
+    execute: async () => {
+      // Get the current notebook's name and path
+      const contentsModel =
+        notebookTracker.currentWidget?.context?.contentsModel;
+      const filePath = contentsModel?.path ?? '';
+      const fileName = contentsModel?.name ?? '';
+
+      // Update the job form inside the notebook jobs widget
+      const newCreateModel = emptyCreateJobModel();
+      newCreateModel.inputFile = filePath;
+      newCreateModel.jobName = fileName;
+
+      await showJobsPanel({
+        jobsView: 'CreateJob',
+        createJobModel: newCreateModel
+      });
+    },
+    label: trans.__('Create a notebook job'),
+    icon: calendarAddOnIcon
+  });
+
+  commands.addCommand(CommandIDs.deleteJob, {
+    execute: async args => {
+      const id = args['id'] as string;
+      await api.deleteJob(id);
+    },
+    // TODO: Use args to name command dynamically
+    label: trans.__('Delete Job')
+  });
+
   commands.addCommand(CommandIDs.stopJob, {
     execute: async args => {
       const id = args['id'] as string;
@@ -225,6 +253,14 @@ async function activatePlugin(
     label: trans.__('Stop Job')
   });
 
+  commands.addCommand(CommandIDs.downloadOutputs, {
+    execute: async args => {
+      const id = args['id'] as string;
+      const redownload = args['redownload'] as boolean;
+      await api.downloadOutputs(id, redownload);
+    }
+  });
+
   // validate presence of status bar
   if (!statusBar) {
     return;
@@ -235,7 +271,7 @@ async function activatePlugin(
     align: 'middle',
     item: ReactWidget.create(
       <RunningJobsIndicator
-        onClick={async () => showJobsPane('ListJobs')}
+        onClick={async () => showJobsPanel({ jobsView: 'ListJobs' })}
         model={scheduledJobsListingModel}
       />
     )
