@@ -15,6 +15,9 @@ import { useTranslator } from '../hooks';
 import { ICreateJobModel, IJobParameter, ListJobsView } from '../model';
 import { Scheduler as SchedulerTokens } from '../tokens';
 
+import ErrorIcon from '@mui/icons-material/Error';
+
+import Alert from '@mui/material/Alert';
 import Button from '@mui/material/Button';
 import Box from '@mui/system/Box';
 import Stack from '@mui/system/Stack';
@@ -71,20 +74,34 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
   // If an error message is "truthy" (i.e., not null or ''), we should display the
   // input in an error state and block form submission.
   const [errors, setErrors] = useState<SchedulerTokens.ErrorsType>({});
+  // Errors for the advanced options
+  const [advancedOptionsErrors, setAdvancedOptionsErrors] =
+    useState<SchedulerTokens.ErrorsType>({});
 
   const api = useMemo(() => new SchedulerService({}), []);
 
   // Retrieve the environment list once.
   useEffect(() => {
     const setList = async () => {
-      setEnvironmentList(await api.getRuntimeEnvironments());
+      const envList = await api.getRuntimeEnvironments();
+      setEnvironmentList(envList);
+      if (envList.length === 1) {
+        props.handleModelChange({
+          ...props.model,
+          environment: envList[0].name
+        });
+      }
     };
 
     setList();
   }, []);
 
   // If any error message is "truthy" (not null or empty), the form should not be submitted.
-  const anyErrors = Object.keys(errors).some(key => !!errors[key]);
+  const anyAdvancedErrors = Object.keys(advancedOptionsErrors).some(
+    key => !!advancedOptionsErrors[key]
+  );
+  const anyErrors =
+    Object.keys(errors).some(key => !!errors[key]) || anyAdvancedErrors;
 
   const handleInputChange = (event: ChangeEvent<HTMLInputElement>) => {
     const target = event.target;
@@ -130,7 +147,10 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
 
   // Takes only a string as input
   const handleTimezoneChange = (value: string | null) => {
-    props.handleModelChange({ ...props.model, timezone: value ?? '' });
+    props.handleModelChange({
+      ...props.model,
+      timezone: value ?? ''
+    });
   };
 
   const handleSelectChange = (event: SelectChangeEvent<string>) => {
@@ -567,10 +587,17 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
       jobOptions.parameters = serializeParameters(props.model.parameters);
     }
 
-    api.createJob(jobOptions).then(response => {
-      // Switch to the list view with "Job List" active
-      props.showListView('Job');
-    });
+    props.handleModelChange({ ...props.model, createError: undefined });
+
+    api
+      .createJob(jobOptions)
+      .then(response => {
+        // Switch to the list view with "Job List" active
+        props.showListView('Job');
+      })
+      .catch((error: string) => {
+        props.handleModelChange({ ...props.model, createError: error });
+      });
   };
 
   const submitCreateJobDefinitionRequest = async (event: React.MouseEvent) => {
@@ -600,10 +627,17 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
       );
     }
 
-    api.createJobDefinition(jobDefinitionOptions).then(response => {
-      // Switch to the list view with "Job Definition List" active
-      props.showListView('JobDefinition');
-    });
+    props.handleModelChange({ ...props.model, createError: undefined });
+
+    api
+      .createJobDefinition(jobDefinitionOptions)
+      .then(response => {
+        // Switch to the list view with "Job Definition List" active
+        props.showListView('JobDefinition');
+      })
+      .catch((error: string) => {
+        props.handleModelChange({ ...props.model, createError: error });
+      });
   };
 
   const removeParameter = (idx: number) => {
@@ -647,12 +681,14 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
   const formPrefix = 'jp-create-job-';
 
   const cantSubmit = trans.__('One or more of the fields has an error.');
+  const createError: string | undefined = props.model.createError;
 
   return (
     <Box sx={{ p: 4 }}>
       <form className={`${formPrefix}form`} onSubmit={e => e.preventDefault()}>
         <Stack spacing={4}>
           <Heading level={1}>Create Job</Heading>
+          {createError && <Alert severity="error">{createError}</Alert>}
           <TextField
             label={trans.__('Job name')}
             variant="outlined"
@@ -687,7 +723,7 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
             id={`${formPrefix}environment`}
             onChange={handleSelectChange}
             environmentList={environmentList}
-            initialValue={props.model.environment}
+            value={props.model.environment}
           />
           <OutputFormatPicker
             label={trans.__('Output formats')}
@@ -726,7 +762,14 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
               id="panel-header"
             >
               <FormLabel component="legend">
-                {trans.__('Additional options')}
+                <Cluster>
+                  {anyAdvancedErrors && (
+                    <ErrorIcon color="error" aria-label="error">
+                      {trans.__('There is an error in the advanced options')}
+                    </ErrorIcon>
+                  )}
+                  {trans.__('Additional options')}
+                </Cluster>
               </FormLabel>
             </AccordionSummary>
             <AccordionDetails id={`${formPrefix}create-panel-content`}>
@@ -734,8 +777,8 @@ export function CreateJob(props: ICreateJobProps): JSX.Element {
                 jobsView={'CreateJob'}
                 model={props.model}
                 handleModelChange={props.handleModelChange}
-                errors={errors}
-                handleErrorsChange={setErrors}
+                errors={advancedOptionsErrors}
+                handleErrorsChange={setAdvancedOptionsErrors}
               />
             </AccordionDetails>
           </Accordion>
