@@ -1,23 +1,37 @@
 import React, { useMemo } from 'react';
-import { IJobDefinitionModel, JobsView, ListJobsView } from '../../model';
+import {
+  IJobDefinitionModel,
+  JobsView,
+  ListJobsView,
+  ICreateJobModel
+} from '../../model';
 import { useTranslator } from '../../hooks';
 import { TextFieldStyled, timestampLocalize } from './job-detail';
 import { SchedulerService } from '../../handler';
 import cronstrue from 'cronstrue';
+import { ListJobsTable } from '../list-jobs';
+import { Scheduler as SchedulerTokens } from '../../tokens';
 
 import {
   Button,
   Card,
   CardContent,
+  FormLabel,
   Stack,
   TextFieldProps
 } from '@mui/material';
+import { ConfirmDeleteButton } from '../../components/confirm-delete-button';
+import { JupyterFrontEnd } from '@jupyterlab/application';
 
 export interface IJobDefinitionProps {
+  app: JupyterFrontEnd;
   model: IJobDefinitionModel;
   refresh: () => void;
   setJobsView: (view: JobsView) => void;
   setListJobsView: (view: ListJobsView) => void;
+  showJobDetail: (jobId: string) => void;
+  showCreateJob: (state: ICreateJobModel) => void;
+  advancedOptions: React.FunctionComponent<SchedulerTokens.IAdvancedOptionsProps>;
 }
 
 export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
@@ -61,13 +75,13 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
           {trans.__('Resume')}
         </Button>
       )}
-      <Button
-        variant="contained"
-        color="error"
-        onClick={handleDeleteJobDefinition}
-      >
-        {trans.__('Delete Job Definition')}
-      </Button>
+      <ConfirmDeleteButton
+        handleDelete={handleDeleteJobDefinition}
+        title={trans.__('Delete Job Definition')}
+        text={trans.__(
+          'Are you sure that you want to delete this job definition?'
+        )}
+      />
     </Stack>
   );
 
@@ -137,10 +151,41 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
     </Card>
   );
 
+  const AdvancedOptions = (
+    <Card>
+      <CardContent>
+        <Stack component="form" spacing={4}>
+          <FormLabel component="legend">
+            {trans.__('Advanced Options')}
+          </FormLabel>
+          <props.advancedOptions
+            jobsView={'JobDetail'}
+            model={props.model}
+            handleModelChange={(_: any) => {
+              return;
+            }}
+            errors={{}}
+            handleErrorsChange={(_: any) => {
+              return;
+            }}
+          />
+        </Stack>
+      </CardContent>
+    </Card>
+  );
+
   const JobsList = (
     <Card>
       <CardContent>
-        <Stack spacing={4}> List of jobs will go here </Stack>
+        <Stack spacing={3}>
+          <ListJobsTable
+            app={props.app}
+            showCreateJob={props.showCreateJob}
+            showJobDetail={props.showJobDetail}
+            jobDefinitionId={props.model.definitionId}
+            pageSize={5}
+          />
+        </Stack>
       </CardContent>
     </Card>
   );
@@ -150,6 +195,7 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
       {DefinitionButtonBar}
       {JobDefinition}
       {JobsList}
+      {AdvancedOptions}
     </>
   );
 }
