@@ -1,4 +1,4 @@
-import React from 'react';
+import React, { useState } from 'react';
 
 import { JupyterFrontEnd } from '@jupyterlab/application';
 import {
@@ -8,8 +8,9 @@ import {
   ListJobsView
 } from '../../model';
 import { useTranslator } from '../../hooks';
-import { SchedulerService } from '../../handler';
+import { Scheduler, SchedulerService } from '../../handler';
 import { Scheduler as SchedulerTokens } from '../../tokens';
+import { ConfirmDeleteButton } from '../../components/confirm-delete-button';
 
 import Button from '@mui/material/Button';
 import Link from '@mui/material/Link';
@@ -22,6 +23,7 @@ import {
   TextField,
   TextFieldProps
 } from '@mui/material';
+import { CommandIDs } from '../..';
 
 export const TextFieldStyled = (props: TextFieldProps): JSX.Element => (
   <TextField
@@ -34,13 +36,12 @@ export const TextFieldStyled = (props: TextFieldProps): JSX.Element => (
 export interface IJobDetailProps {
   app: JupyterFrontEnd;
   model: IJobDetailModel;
-  handleModelChange: () => void;
+  handleModelChange: () => Promise<void>;
   setCreateJobModel: (createModel: ICreateJobModel) => void;
   setJobsView: (view: JobsView) => void;
   setListJobsView: (view: ListJobsView) => void;
   // Extension point: optional additional component
   advancedOptions: React.FunctionComponent<SchedulerTokens.IAdvancedOptionsProps>;
-  outputFormatsStrings?: string[];
 }
 
 export const timestampLocalize = (time: number | ''): string => {
@@ -57,6 +58,7 @@ export const timestampLocalize = (time: number | ''): string => {
 
 export function JobDetail(props: IJobDetailProps): JSX.Element {
   const trans = useTranslator('jupyterlab');
+  const [downloading, setDownloading] = useState(false);
 
   const ss = new SchedulerService({});
 
@@ -91,8 +93,28 @@ export function JobDetail(props: IJobDetailProps): JSX.Element {
     props.handleModelChange();
   };
 
+  const downloadOutputs = async () => {
+    setDownloading(true);
+    await props.app.commands.execute(CommandIDs.downloadOutputs, {
+      id: props.model.jobId,
+      redownload: false
+    });
+    await new Promise(res => setTimeout(res, 500));
+    await props.handleModelChange();
+    setDownloading(false);
+  };
+
   const ButtonBar = (
     <Stack direction="row" gap={2} justifyContent="flex-end" flexWrap={'wrap'}>
+      {props.model.downloaded === false && props.model.status === 'COMPLETED' && (
+        <Button
+          variant="outlined"
+          onClick={downloadOutputs}
+          disabled={downloading}
+        >
+          {trans.__('Download Output Files')}
+        </Button>
+      )}
       {props.model.status === 'IN_PROGRESS' && (
         <Button variant="outlined" onClick={handleStopJob}>
           {trans.__('Stop Job')}
@@ -101,9 +123,11 @@ export function JobDetail(props: IJobDetailProps): JSX.Element {
       <Button variant="outlined" onClick={handleRerunJob}>
         {trans.__('Rerun Job')}
       </Button>
-      <Button variant="contained" color="error" onClick={handleDeleteJob}>
-        {trans.__('Delete Job')}
-      </Button>
+      <ConfirmDeleteButton
+        handleDelete={handleDeleteJob}
+        title={trans.__('Delete Job')}
+        text={trans.__('Are you sure that you want to delete this job?')}
+      />
     </Stack>
   );
 
@@ -152,16 +176,14 @@ export function JobDetail(props: IJobDetailProps): JSX.Element {
   ];
 
   function OutputFile(props: {
-    outputType: string;
+    output: Scheduler.IOutput;
     app: JupyterFrontEnd;
-    outputPath: string;
   }) {
-    const outputName = props.outputPath.replace(/ipynb$/, props.outputType);
     return (
       <Link
-        key={props.outputType}
-        href={`/lab/tree/${outputName}`}
-        title={trans.__('Open "%1"', outputName)}
+        key={props.output.output_format}
+        href={`/lab/tree/${props.output.output_path}`}
+        title={trans.__('Open "%1"', props.output.output_path)}
         onClick={(
           e:
             | React.MouseEvent<HTMLSpanElement, MouseEvent>
@@ -169,16 +191,24 @@ export function JobDetail(props: IJobDetailProps): JSX.Element {
         ) => {
           e.preventDefault();
           props.app.commands.execute('docmanager:open', {
-            path: outputName
+            path: props.output.output_path
           });
         }}
         style={{ paddingRight: '1em' }}
       >
-        {outputName}
+        {props.output.display_name}
       </Link>
     );
   }
 
+  const convertJsonToOutput = (output: { [key: string]: string }) => {
+    return {
+      display_name: output['display_name'],
+      output_format: output['output_format'],
+      output_path: output['output_path'] || null
+    } as Scheduler.IOutput;
+  };
+
   const CoreOptions = (
     <Card>
       <CardContent>
@@ -200,13 +230,15 @@ export function JobDetail(props: IJobDetailProps): JSX.Element {
               <FormLabel component="legend">
                 {trans.__('Output files')}
               </FormLabel>
-              {props.outputFormatsStrings?.map(outputFormatString => (
-                <OutputFile
-                  outputType={outputFormatString}
-                  app={props.app}
-                  outputPath={props.model.outputPath}
-                />
-              ))}
+              {props.model.outputs.map(
+                output =>
+                  output['output_path'] && (
+                    <OutputFile
+                      output={convertJsonToOutput(output)}
+                      app={props.app}
+                    />
+                  )
+              )}
             </>
           )}
         </Stack>
@@ -255,11 +287,11 @@ export function JobDetail(props: IJobDetailProps): JSX.Element {
           <props.advancedOptions
             jobsView={'JobDetail'}
             model={props.model}
-            handleModelChange={model => {
+            handleModelChange={(_: any) => {
               return;
             }}
             errors={{}}
-            handleErrorsChange={errors => {
+            handleErrorsChange={(_: any) => {
               return;
             }}
           />
